package com.dgtly.wxportal.controller;

import com.dgtly.common.annotation.ApiPassToken;
import com.dgtly.common.core.controller.ApiBaseController;
import com.dgtly.common.core.domain.AjaxResult;
import com.dgtly.common.core.domain.ParameterObject;
import com.dgtly.system.domain.CustomersExt;
import com.dgtly.system.domain.SysUser;
import com.dgtly.system.service.ICustomersExtService;
import com.dgtly.system.service.ISysUserExtService;
import com.dgtly.system.service.ISysUserService;
import com.dgtly.wxportal.domain.OrderFile;
import com.dgtly.wxportal.service.*;
import com.sun.org.apache.xpath.internal.operations.Bool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * @description:e签宝认证
 * @author:qxm
 * @date:2020/10/12 16:18
 */
@Api(tags = "e签宝相关API", description = "提供e签宝服务API")
@RequestMapping("eSign")
@RestController
@ApiPassToken
public class EsignController extends ApiBaseController {

    @Autowired
    private IEsignPersonRealnVerifyService personRealnVerifyService;
    @Autowired
    private IEsignOrganRealVerifyService organRealVerifyService;
    @Autowired
    private IEsignSignService signService;
    @Autowired
    private IOrderFileService orderFileService;
    @Autowired
    private ISysUserService userService;
    @Autowired
    private ISysUserExtService userExtService;
    @Autowired
    private ICustomersExtService customersExtService;
    @Autowired
    private ESignService eSignService;

    /**
     * 经销商订单签收方法
     * 没有注册就注册，注册完就认证，然后再点击签署按钮，获取经销商用户id，对应主体id
     * E签宝一步发起签署
     */
    @ApiOperation(value = "订单签收", notes = "参数：{'orderId':'xxx','userId':'xxx'}" +
            "\n（orderId：订单编号" +
            "\n（userId：用户id")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params", paramType = "body")
    })
    @PostMapping("dealerSign")
    public Object dealerSign() {

        ParameterObject obj = getParameterObject();
        obj.checkParameterNotNull("orderId,userId");
        String orderId = obj.getString("orderId");
        String userId = obj.getString("userId");

        String businessScene = "订单签收";
        String fileName = "签收订单.pdf";
        /*根据userId查询个人注册后accountId*/
        String signerAccountId = "";
        SysUser sysUser = userService.selectUserById(Long.parseLong(userId));
        if (sysUser!=null){
            signerAccountId=sysUser.getAccountId();
        }
        /*根据订单id关联查询企业注册后id*/
        String authorizedAccountId =customersExtService.selectOrgIdByOrderId(orderId);

        /*首先根据订单id-orderId查询订单是否已被签署*/
        OrderFile orderFlag = orderFileService.selectOrderFileById(orderId);
        if (orderFlag==null) {
            /*1、根据传入订单id，查询订单相关信息生成PDF，调用e签宝上传方法返回文件fileId*/
            String fileId = eSignService.upload(orderId);
            /*2、调用E签宝接口一步发起流程，得到返回的流程id*/
            String flowId = signService.oneStepFlow(businessScene, fileId, fileName, signerAccountId, authorizedAccountId);
            /*3、调用E签宝获取签署地址*/
            String result = signService.processExecuteUrl(flowId, signerAccountId, authorizedAccountId);
            /*4、保存订单id和flowId*/
            OrderFile orderFile = new OrderFile();
            orderFile.setOrderId(orderId);
            orderFile.setCustomersCode(authorizedAccountId);
            orderFile.setSignerId(signerAccountId);
            orderFile.setSignerName("");
            orderFile.setFlowId(flowId);
            orderFileService.insertOrderFile(orderFile);
            return result;
        }else{
            return AjaxResult.error("订单已签署");
        }
    }

    /**
     * 经销商通过签署地址签署后，回调此方法，通过flowId得到签署后的文件下载地址并保存url
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params", paramType = "body")
    })
    @PostMapping("updateOrderFile")
    public Object updateOrderFile() {
        ParameterObject obj = getParameterObject();
        obj.checkParameterNotNull("flowId");
        String flowId = obj.getString("flowId");
        String fileUrl = signService.processDocumentDownload(flowId);

        OrderFile orderFile = new OrderFile();
        orderFile.setFlowId(flowId);
        orderFile.setFileUrl(fileUrl);
        orderFileService.updateOrderFileByFlowId(orderFile);
        return fileUrl;
    }


    @ApiOperation(value = "E签宝个人认证", notes = "参数：{'userId':'XXXXXXX'}" +
            "\n（userId：待认证用户id）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params", paramType = "body")
    })
    @PostMapping("personRealnVerify")
    public Object personRealnVerify() {
        ParameterObject obj = getParameterObject();
        obj.checkParameterNotNull("userId");
        String userId = obj.getString("userId");
        SysUser sysUser = userService.selectUserById(Long.parseLong(userId));
        String result="";
        if (sysUser!=null){
            result = personRealnVerifyService.personRealnVerify(sysUser.getAccountId());
        }
        return result;
    }

    @ApiOperation(value = "E签宝个人认证成功回调接口（修改认证状态）",
                notes = "参数：{" +
                        "userId:'123'}" +
                        "错误：" +
                        "500 修改个人认证状态失败" +
                        "301 个人认证失败" +
                        "")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params" , paramType = "body")
    })
    @PostMapping("/updateUserIsAuthentication")
    public Object updateUserIsAuthentication(){
        ParameterObject obj =  getParameterObject();
        String accountId = obj.getString("accountId");
        Boolean success = obj.getBoolean("success");
        if (success) {
            int result = userService.updateIsAuthentication("1", accountId);
            if (result > 0) {
                return AjaxResult.success().putKV("result", result);
            } else {
                return AjaxResult.error(500, "修改个人认证状态失败");
            }
        }else {
            return AjaxResult.error(301,"个人认证失败");
        }
    }


    @ApiOperation(value = "E签宝企业认证", notes = "参数：{'userId':'XXXXXXX'}" +
            "\n（userId：待认证e签宝用户id）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params", paramType = "body")
    })
    @PostMapping("organRealVerify")
    public Object organRealVerify() {
        ParameterObject obj = getParameterObject();
        obj.checkParameterNotNull("userId");
        String userId = obj.getString("userId");
        SysUser sysUser = userService.selectUserById(Long.parseLong(userId));
        String result="";
        if (sysUser!=null){
            CustomersExt customersExt = customersExtService.selectCustomersExtById(sysUser.getSysUserExt().getOrgCode());
            if (customersExt!=null){
                result = organRealVerifyService.organRealVerify(sysUser.getAccountId(), customersExt.getOrgId());
            }
        }
        return result;
    }

    /*调用修改数据库企业认证状态*/
    @ApiOperation(value = "E签宝企业认证成功回调接口（修改认证状态）",
            notes = "参数：{" +
                    "userId:'123'}" +
                    "错误：" +
                    "500 修改企业认证状态失败" +
                    "301 企业认证失败" +
                    "")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params" , paramType = "body")
    })
    @PostMapping("/updateCustomersIsAuthentication")
    public Object updateCustomersIsAuthentication(){
        ParameterObject obj =  getParameterObject();
        String accountId = obj.getString("accountId");
        Boolean success = obj.getBoolean("success");
        if (success) {
            int result = customersExtService.updateCustomersIsAuthentication("1", accountId);
            if (result > 0) {
                return AjaxResult.success().putKV("result", result);
            } else {
                return AjaxResult.error(500, "修改企业认证状态失败");
            }
        }else {
            return AjaxResult.error(301,"企业认证失败");
        }
    }

}
